/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("net.jazz.ajax.ui.internal.HoverPopup");

dojo.require("dijit.Tooltip");

(function() {

/*
 * The HoverPopup is a modified version of dijit.Tooltip. The difference is that the HoverPopup will
 * stay open as long as the cursor is over either the connected node, or the tooltip itself (rather than just the
 * connected node). This allows more sophisticated interaction in the tooltip than static information. Users can
 * follow links in the tooltip, etc.
 * 
 * One important limitation to note is that the innerHTML of the provided domNode is used to populate the tooltip.
 * This is because, behind the scenes, there is actually one master tooltip that is reused for each individal tooltip.
 * When it is displayed, the required HTML is copied out of the desired tooltip. Also note that because the innerHTML
 * is used, css classes or properties applied to the root node will be lost. To workaround this, add a child node to the
 * root that has the required attributes and put your content in it.
 * 
 * HoverPopups should be created as follows: new HoverPopup(args, domNode);
 * 	- the domNode param gives a reference to the domNode who's innerHTML should be used to populate the tooltip. In
 * 		the case of lazily populated tooltips it can be empty initially and populated when the callback is triggered.
 * 	- the args param should be an object that contains any or all of the following attributes:
 * 		- connectId: This is an array of domNode ids that this tooltip should be connected to. Hovering the referenced nodes
 * 			will show the tooltip around them. This attribute or the connectNode attribute must be given (or your tooltip
 * 			will not be attached to anything), but both may be used together. NOTE: if connectId is used, the referenced
 * 			node must be in the document hierarchy before the tooltip is instantiated.
 * 		- connectNode: This is an array of references to domNodes that this tooltip should be connected to. Hovering the
 * 			referenced will show the tooltip around them. This attribute or the connectId attribute must be given (or your
 * 			tooltip will not be attached to anything), but both may be used together.
 * 		- isImagePreview: set this parameter to true to style the tooltip as an image preview (blue borders and backgrounds).
 * 		- isProgrammatic: indicates that the tooltip will always be programatically controlled by the client. That is
 * 			to say that it will never be opened or closed by hovering (connectNodes need not be supplied). Clients should
 * 			call HoverPopup#open(<target_node>) and HoverPopup#close().
 * 		- callback: This is a function that will be called immediately before the tooltip is displayed. As soon as this
 * 			method returns the tooltip is shown. For this reason, if the tooltip is populated based on the results of an
 * 			asynchronous call, the tooltip must be refreshed (HoverPopup.refresh()) after the html is ready.
 * 		- callbackArg: This is a single object with any number of attributes. It will be passed to the function specified
 * 			in the callback parameter, but will have one additional property added to it called "tooltip" which will be a
 * 			a reference to the tooltip widget that is calling it. This allows the callback function to call, for example,
 * 			args.tooltip.refresh() when it is done any asynchronous calls.
 * 		- repopulateDelay: This gives the number of milliseconds to wait between subsequent displays of the tooltip before
 * 			calling the populate callback again. This allows clients to reuse the cached tooltip content and decide how
 * 			regularly new information should be retreived from the server. A value of -1 indicates that once the tooltip is
 * 			populated, it should never be populated again.
 * 		- preDisplayCheck: This is a function that will be called before showing the tooltip. It should return a boolean
 * 			result which indicates if the tooltip should be displayed or not. This allows the tooltip to be supressed under
 * 			certain circumstances.
 * 		- closeCallback: This is a function that will be called when the tooltip is closed. Clients can use this to perform
 * 			any required cleanup.
 * 		- orientation: This params allows the client to override the tooltip positioning logic and explicitly indicate where
 * 			the tip should be opened. It should be a string that is 1 or 2 characters long. Each character represents a direction.
 * 			'L' means left, 'R' means right, 'T' means top, 'B' means bottom (e.g. "TR" means open the tooltip to the right
 * 			of the node and align the top of the tip with the top of the node, "L" means open the tooltip to the left of the node
 * 			and calculate vertical orientation dynamically). Note that if there isn't room in the specified direction, the hint
 * 			will be ignored.
 * 		- label: This attribute can be used in place of the second constructor argument when the tooltip contains static content.
 * 			If specified, the label attribute will be used as innerHTML to populate the tooltip instead of the contents of the
 * 			specified domNode.
 */

dojo.declare("net.jazz.ajax.ui.internal.HoverPopup", dijit.Tooltip, {
	
	hideDelay: 190,
	
	showDelay: 600,
	
	postCreate: function(){
		if (!this.callbackArg)
			this.callbackArg = new Object();
		this.callbackArg.tooltip = this;
		
		if(this.srcNodeRef){
			this.srcNodeRef.style.display = "none";
		}

		this._connectNodes = [];
		
		dojo.forEach(this.connectId, function(id) {
			var node = dojo.byId(id);
			if (node) {
				this._setupConnectNode(node);
			}
		}, this);
		dojo.forEach(this.connectNode, function(node) {
			this._setupConnectNode(node);
		}, this);
	},
	
	_setupConnectNode: function(node) {
		this._connectNodes.push(node);
		dojo.forEach(["onMouseOver", "onMouseOut", "onFocus", "onBlur", "onHover", "onUnHover"], function(event){
			this.connect(node, event.toLowerCase(), "_"+event);
		}, this);
		if(dojo.isIE){
			// BiDi workaround
			node.style.zoom = 1;
		}
	},
	
	open: function(/*DomNode*/ target) {
		if (this.preDisplayCheck && !this.preDisplayCheck())
			return;
		this._eventHandles = new Array();
		dijit.checkTooltip();
		dojo.forEach(["onMouseOver", "onHover", "onMouseOut", "onUnHover"], function(event){
				this._eventHandles.push(this.connect(dijit._masterTT.domNode, event.toLowerCase(), "_"+event+"Tooltip"));
			}, this);
		if (this.callback) {
			var populate = true;
			if (this.repopulateDelay) {
				var date = new Date();
				var time = date.getTime();
				if (this._timeStamp && (this.repopulateDelay == -1 || this._timeStamp + this.repopulateDelay > time))
					populate = false;
				else
					this._timeStamp = time;
			}
			if (populate)
				this.callback(this.callbackArg);
		}
		if (this.isImagePreview) {
			dojo.addClass(dijit._masterTT.domNode, "dijitTooltipBlue");
			dojo.addClass(dijit._masterTT.connectorWrapper, "dijitTooltipBlue");
		} else if (this.isMenu) {
			dojo.addClass(dijit._masterTT.domNode, "dijitTooltipMenu");
		}
		if (this.orientation)
			dijit._masterTT.orientation = this.orientation;
		try {
			this.inherited(arguments, [this._determineTarget(target)]);
		} catch (e){
			// Swallow errors
		}
		this.validateConnectNode();
	},
	
	validateConnectNode: function() {
		if (dojo.isDescendant(this._connectNode, document.body)) {
			this._validateNodeTimeout = setTimeout(dojo.hitch(this,this.validateConnectNode),500);
			return;
		}
		this.close();
			
	},
	
	close: function() {
		if (this._validateNodeTimeout) {
			clearTimeout(this._validateNodeTimeout)
			delete this._validateNodeTimeout;
		}
		if (!this._connectNode)
			return;
		this.inherited(arguments);
		if (this.isImagePreview) {
			dojo.removeClass(dijit._masterTT.domNode, "dijitTooltipBlue");
			dojo.removeClass(dijit._masterTT.connectorWrapper, "dijitTooltipBlue");
		} else if (this.isMenu) {
			dojo.removeClass(dijit._masterTT.domNode, "dijitTooltipMenu");
		}
		delete dijit._masterTT.orientation;
		if (this._eventHandles) {
			this.disconnect(this._eventHandles);
			this._eventHandles = null;
		}
		if (this.closeCallback) {
			this.closeCallback();
		}
	},
	
	refresh: function() {
		if (dojo.isDescendant(this._connectNode, document.body)) {
			dijit.showTooltip(this.label || this.domNode.innerHTML, this._connectNode);
			if (!this._validateNodeTimeout)
				this.validateConnectNode();
		}
	},
	
	_determineTarget: function(/*DomNode*/ target) {
		if (!target)
			return target;
		var newTarget = null;
		for (var i = 0; i < this._connectNodes.length; i++) {
			if (!this._connectNodes[i])
				continue;
			// if the target is this connect node, return it
			if (this._connectNodes[i] == target)
				return target;
			// if the target is a descendant of this connect node
			// make sure we return the lowest level connect node that is a parent of the target
			// (e.g. as close to the target as possible)
			if (dojo.isDescendant(target, this._connectNodes[i]) == true &&
					(!newTarget || dojo.isDescendant(this._connectNodes[i], newTarget) == true)) {
				newTarget = this._connectNodes[i];
			}
		}
		// return the suitable parent if one was found or the original target otherwise
		return newTarget ? newTarget : target;
	},
	
	_onMouseOverTooltip: function(e) {
		this._onHoverTooltip(e);
	},
	
	_onHoverTooltip: function(e) {
		this.isTipHovered = true;
	},
	
	_onMouseOutTooltip: function(e) {
		if(dojo.isDescendant(e.relatedTarget, dijit._masterTT.domNode) == true){
			// false event; just moved from target to target child; ignore.
			return;
		}
		this._onUnHoverTooltip(e);
	},
	
	_onUnHoverTooltip: function(e) {
		this.isTipHovered = false;
		if (e.relatedTarget == this._connectNode || dojo.isDescendant(e.relatedTarget, this._connectNode) == true) {
			// moved from tooltip back to connectNode; ignore.
			return;
		}
		this._onUnHover(e);
	},

	// this function is only overriden as a workaround to: http://trac.dojotoolkit.org/ticket/5464
	_onMouseOut: function(/*Event*/ e){
		// need to use == true since isDescendant returns -1 when it catches an exception
		if(dojo.isDescendant(e.relatedTarget, e.target) == true){
			// false event; just moved from target to target child; ignore.
			return;
		}
		this._onUnHover(e);
	},
	
	_onHover: function(e) {
		if (this.isProgrammatic || this.isMenu)
			return;
		if (this._unHoverTimer) {
			clearTimeout(this._unHoverTimer);
			delete this._unHoverTimer;
		}
		this.inherited(arguments);
	},
	
	_onUnHover: function(e) {
		if (this.isProgrammatic)
			return;
		this._unHoverTimer = setTimeout(dojo.hitch(this,function(){
			if (!this.isTipHovered) {
				if(this._showTimer){
					clearTimeout(this._showTimer);
					delete this._showTimer;
				}
				this.close();
			}
			delete this._unHoverTimer;
		}), this.hideDelay);
	}
	
});

})();

dijit.checkTooltip = function() {
	if(!dijit._masterTT){ dijit._masterTT = new dijit._MasterTooltip(); }
};

dojo.addOnLoad(function(){
	
	// for some reason dojo.moduleUrl refuses to work in an onload...?
	// have to override the template with a string
	dijit._MasterTooltip.prototype.templateString =
		"<div class='dijitTooltip dijitTooltipLeft' id='dojoTooltip'>" +
		"	<div class='dijitTooltipWrapper'>" +
		"		<div class='dijitTooltipTopWrapper'>" +
		"			<div class='dijitTooltipTrimWrapperRel'>" +
		"				<div class='dijitTooltipTopTrim'></div>" +
		"			</div>" +
		"		</div>" +
		"		<div class='dijitTooltipContainer dijitTooltipContents' dojoAttachPoint='containerNode' waiRole='alert'>" +
		"			<div class='dijitTooltipSideTrim'>" +
		"				<div class='dijitTooltipInnerContentWrapper' dojoAttachPoint='innerContentWrapper'>" +
		"					<div dojoAttachPoint='contentNode' class='dijitTooltipInnerContent'>" +
		"					</div>" +
		"				</div>" +
		"			</div>" +
		"		</div>" +
		"		<div class='dijitTooltipBottomWrapper'>" +
		"			<div class='dijitTooltipTrimWrapperRel'>" +
		"				<div class='dijitTooltipBottomTrim'></div>" +
		"			</div>" +
		"		</div>" +
		"		<div class='dijitTooltipCornerBL'></div>" +
		"		<div class='dijitTooltipCornerBR'></div>" +
		"		<div class='dijitTooltipCornerTL'></div>" +
		"		<div class='dijitTooltipCornerTR'></div>" +
		"	</div>" +
		"	<div dojoAttachPoint='connectorWrapper'><div class='dijitTooltipConnector'></div></div>" +
		"</div>";
	
	// override the dijit._MasterTooltip.show() function
	dijit._MasterTooltip.prototype.show = function(/*String*/ innerHTML, /*DomNode*/ aroundNode){
		// summary:
		//	Display tooltip w/specified contents to right specified node
		//	(To left if there's no space on the right, or if LTR==right)

		this.contentNode.innerHTML=innerHTML;
		
		// Firefox bug. when innerHTML changes to be shorter than previous
		// one, the node size will not be updated until it moves.  
		this.domNode.style.top = (this.domNode.offsetTop + 1) + "px";
		
		// reset height in IE6 so CSS expression will work
		if (dojo.isIE && dojo.isIE < 7) {
			this.innerContentWrapper.style.height = "1px";
			this.innerContentWrapper.style.height = "";
		}
		
		if (dojo.hasClass(this.domNode, "dijitTooltipBlue")) {
			var width = document.documentElement.clientWidth * 0.8;
			var height = document.documentElement.clientHeight * 0.8;
			this.innerContentWrapper.style.maxWidth = width + "px";
			this.innerContentWrapper.style.maxHeight = height + "px";
		} else {
			this.innerContentWrapper.style.maxWidth = "";
			this.innerContentWrapper.style.maxHeight = "";
		}
		
		// position the element and change CSS according to position
		var align = this.getAlign(aroundNode);
		if (!align)
			return;
		this.connectorWrapper.style.display = "";
		var pos = dijit.placeOnScreenAroundElement(this.domNode, aroundNode, align);
		
		// adjust position if necessary
		var viewportHeight = document.documentElement.clientHeight;
		var tipCoords = dojo.coords(this.domNode,false);
		var y = tipCoords.y;
		var connector = dojo.query(".dijitTooltipConnector",this.domNode)[0];
		
		if (pos.corner=='TL' || pos.corner=='BL') {
			dojo.addClass(this.domNode, "dijitTooltipRight");
			dojo.removeClass(this.domNode, "dijitTooltipLeft");
		} else {
			dojo.addClass(this.domNode, "dijitTooltipLeft");
			dojo.removeClass(this.domNode, "dijitTooltipRight");
		}
		if (pos.corner=='TL' || pos.corner=='TR') {
			dojo.addClass(connector, "dijitTooltipConnectorTop");
			dojo.addClass(this.containerNode, "dijitTooltipContainerTop");
			
			// Adjust position for custom style
			var offset = 0;
			while (y + tipCoords.h - offset > viewportHeight-5 && y - offset - 3 > 5) {
				offset++;
			}
			this.domNode.style.top = (pos.y - offset - 3) + "px";
		} else {
			dojo.removeClass(connector, "dijitTooltipConnectorTop");
			dojo.removeClass(this.containerNode, "dijitTooltipContainerTop");
			
			// Adjust position for custom style
			this.domNode.style.top = pos.y + 10 + "px";
		}
		// ensure we are using as much space as possible if this is an image preview
		if (dojo.hasClass(this.domNode, "dijitTooltipBlue") && this.domNode.clientWidth < width) {
			var start = pos.x + 3;
			this.domNode.style.left = start + "px";
			var offset = 3;
			do {
				var oldWidth = this.domNode.clientWidth;
				this.domNode.style.left = start - offset + "px"; 
				offset += 3;
			} while (oldWidth < this.domNode.clientWidth && this.domNode.clientWidth <= width);
		}
		
		// determine if we should show the connector (don't if the node is hidden by the tooltip)
		// don't show it if we are using a menu
		var nodeCoords = dojo.coords(aroundNode, true);
		var tipCoords = dojo.coords(this.domNode, true);
		if (dojo.hasClass(this.domNode, "dijitTooltipMenu") ||
				tipCoords.y + tipCoords.h < nodeCoords.y || tipCoords.y > nodeCoords.y + nodeCoords.h ||
				(dojo.hasClass(this.domNode, "dijitTooltipLeft") && tipCoords.x +tipCoords.w > nodeCoords.x + nodeCoords.w) ||
				(dojo.hasClass(this.domNode, "dijitTooltipRight") && tipCoords.x < nodeCoords.x))
			this.connectorWrapper.style.display = "none";
		else {
			var midpoint = nodeCoords.y + nodeCoords.h/2 - 5;
			connector.style.top = midpoint-tipCoords.y + "px";
		}
		
		
		this.isShowingNow = true;
		this.aroundNode = aroundNode;
	};
	
	// determine where the node is in the viewport and generate the align param for dijit.placeOnScreenAroundElement accordingly
	//   if aroundNode is in the right half of the viewport, try to place the tooltip on the left and vice versa
	//   if aroundNode is in the top half of the viewport, try to place the tooltip on the bottom and vice versa
	dijit._MasterTooltip.prototype.getAlign = function(/*DomNode*/ aroundNode){
		var aroundNodePos = dojo.coords(aroundNode, true);
		// if all the coordinates are 0, the node no longer exists and the show should be cancelled
		if (aroundNodePos.h == 0 && aroundNodePos.l == 0 && aroundNodePos.t == 0 && aroundNodePos.w == 0 && aroundNodePos.x == 0 && aroundNodePos.y == 0)
			return false;
			
		var scrollTop = self.pageYOffset;
		if (!scrollTop)
			scrollTop = document.body.scrollTop;
		var scrollLeft = self.pageXOffset;
		if (!scrollLeft)
			scrollLeft = document.body.scrollLeft;
		
		var forceLeft = this.orientation ? this.orientation.indexOf("L") != -1 : false;
		var forceRight = this.orientation ? this.orientation.indexOf("R") != -1 : false;
		var forceTop = this.orientation ? this.orientation.indexOf("T") != -1 : false;
		var forceBottom = this.orientation ? this.orientation.indexOf("B") != -1 : false;
		
		var invertRightLeft = forceLeft || (!forceRight  && (document.body.offsetWidth - ((aroundNodePos.x + (aroundNodePos.w / 2)) - scrollLeft) < document.body.offsetWidth / 2));
		invertRightLeft = invertRightLeft ? !this.isLeftToRight() : this.isLeftToRight();
		if (forceBottom  || (!forceTop && (document.body.offsetHeight - ((aroundNodePos.y + (aroundNodePos.h / 2)) - scrollTop) < document.body.offsetHeight / 2))) {
			if (dojo.hasClass(this.domNode, "dijitTooltipMenu"))
				return invertRightLeft ? {'TL': 'BL', 'TR': 'BR'} : {'TR': 'BR', 'TL': 'BL'};
			return invertRightLeft ? {'BR': 'BL', 'BL': 'BR'} : {'BL': 'BR', 'BR': 'BL'};
		}
		if (dojo.hasClass(this.domNode, "dijitTooltipMenu"))
			return invertRightLeft ? {'BL': 'TL', 'BR': 'TR'} : {'BR': 'TR', 'BL': 'TL'};
		return invertRightLeft ? {'TR': 'TL', 'TL': 'TR'} : {'TL': 'TR', 'TR': 'TL'};
	};
	
	dijit._MasterTooltip.prototype.hide = function(aroundNode){
		// summary: hide the tooltip
		if(!this.aroundNode || this.aroundNode !== aroundNode){
			return;
		}
		this.isShowingNow = false;
		this.aroundNode = null;
		this._onHide();
	};

});